@@ -1,4 +1,4 @@
-# Copyright (c) Microsoft Corporation. All rights reserved.
+# Copyright (c) Microsoft Corporation and Fairlearn contributors.
 # Licensed under the MIT License.
 
 import nbformat as nbf
@@ -76,19 +76,6 @@ def assay_one_notebook(notebook_name, test_values):
         assert nb.scraps[k].data == v.expected
 
 
-@pytest.mark.notebooks
-def test_group_metrics_notebook():
-    overall_recall_key = "overall_recall"
-    by_groups_key = "recall_by_groups"
-
-    test_values = {}
-    test_values[overall_recall_key] = ScrapSpec("group_metrics.overall", 0.5)
-    test_values[by_groups_key] = ScrapSpec(
-        "results.by_group", {'a': 0.0, 'b': 0.5, 'c': 0.75, 'd': 0.0})
-
-    assay_one_notebook("Group Metrics", test_values)
-
-
 @pytest.mark.notebooks
 def test_grid_search_for_binary_classification():
     nb_name = "Grid Search for Binary Classification"
@@ -97,33 +84,12 @@ def test_grid_search_for_binary_classification():
     test_values["best_lambda_second_grid"] = ScrapSpec(
         "lambda_best_second", pytest.approx(0.8333333333))
     test_values["best_coeff_second0"] = ScrapSpec(
-        "second_sweep._predictors[second_sweep._best_grid_index].coef_[0][0]",
+        "second_sweep.predictors_[second_sweep.best_idx_].coef_[0][0]",
         pytest.approx(2.53725364))
 
     assay_one_notebook(nb_name, test_values)
 
 
-@pytest.mark.notebooks
-def test_binary_classification_on_compas_dataset():
-    nb_name = "Binary Classification on COMPAS dataset"
-
-    test_values = {}
-    test_values["pp_eo_aa_pignore"] = ScrapSpec(
-        "postprocessed_predictor_EO._post_processed_predictor_by_sensitive_feature['African-American']._p_ignore",  # noqa: E501
-        pytest.approx(0.2320703126)
-    )
-
-    assay_one_notebook(nb_name, test_values)
-
-
-@pytest.mark.notebooks
-def test_grid_search_with_census_data():
-    nb_name = "Grid Search with Census Data"
-    test_values = {}
-    test_values["len_nondominated"] = ScrapSpec("len(non_dominated)", 13)
-    assay_one_notebook(nb_name, test_values)
-
-
 @pytest.mark.notebooks
 def test_mitigating_disparities_in_ranking_from_binary_data():
     nb_name = "Mitigating Disparities in Ranking from Binary Data"
@@ -139,4 +105,12 @@ def test_mitigating_disparities_in_ranking_from_binary_data():
 def test_binary_classification_with_the_uci_credit_card_default_dataset():
     nb_name = "Binary Classification with the UCI Credit-card Default Dataset"
     test_values = {}
+    test_values["Contains_Unmitigated"] = ScrapSpec(
+        "'Unmitigated' in model_sweep_dict",
+        True
+    )
+    test_values["Contains_ThresholdOptimizer"] = ScrapSpec(
+        "'ThresholdOptimizer' in model_sweep_dict",
+        True
+    )
     assay_one_notebook(nb_name, test_values)
