@@ -1,9 +1,23 @@
-# Copyright (c) Microsoft Corporation. All rights reserved.
+# Copyright (c) Microsoft Corporation and Fairlearn contributors.
 # Licensed under the MIT License.
 
 import pytest
 
 import fairlearn.metrics as metrics
+from fairlearn.metrics._selection_rate import _EMPTY_INPUT_PREDICTIONS_ERROR_MESSAGE
+
+
+def test_selection_rate_empty():
+    with pytest.raises(ValueError) as exc:
+        _ = metrics.selection_rate([], [])
+    assert _EMPTY_INPUT_PREDICTIONS_ERROR_MESSAGE == exc.value.args[0]
+
+
+def test_selection_rate_single_element():
+    assert 1 == metrics.selection_rate([1], [1])
+    assert 1 == metrics.selection_rate([0], [1])
+    assert 0 == metrics.selection_rate([1], [0])
+    assert 0 == metrics.selection_rate([0], [0])
 
 
 def test_selection_rate_unweighted():
@@ -34,25 +48,3 @@ def test_selection_rate_non_numeric():
     result = metrics.selection_rate(y_true, y_pred, pos_label=b)
 
     assert result == 0.375
-
-
-def test_selection_rate_group_summary():
-    a = "a"
-    b = "b"
-    q = "q"
-    r = "r"
-
-    y_true = [a, b, a, b, b, b, a, a]
-    y_pred = [a, b, a, a, b, b, a, b]
-    weight = [1, 2, 3, 4, 1, 2, 4, 8]
-    groups = [r, r, r, r, q, q, q, q]
-
-    result = metrics.selection_rate_group_summary(
-        y_true, y_pred,
-        sensitive_features=groups,
-        pos_label=b,
-        sample_weight=weight)
-
-    assert result.overall == 0.52
-    assert result.by_group[r] == 0.2
-    assert result.by_group[q] == pytest.approx(0.73333333)
